//===================================================================================
// Microsoft patterns & practices
// Composite Application Guidance for Windows Presentation Foundation and Silverlight
//===================================================================================
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===================================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===================================================================================
using System;
using Microsoft.Practices.Prism.Regions.Behaviors;
using Windows.UI.Xaml;
//using Prism.Desktop.Metro;

namespace Microsoft.Practices.Prism.Regions
{
    /// <summary>
    /// Provides a base class for region's behaviors.
    /// </summary>
    public abstract class RegionBehavior : IRegionBehavior
    {
        private IRegion region;

        /// <summary>
        /// Behavior's attached region.
        /// </summary>
        public IRegion Region
        {
            get
            {
                return region;
            }
            set
            {
                if (this.IsAttached)
                {
                    throw new InvalidOperationException(ResourceHelper.RegionBehaviorRegionCannotBeSetAfterAttach);
                }

                this.region = value;
            }
        }


        private WeakReference elementWeakReference;

        /// <summary>
        /// The element that will host the Region. 
        /// </summary>
        /// <value>The target element.</value>
        public DependencyObject HostControl
        {
            get { return this.elementWeakReference != null ? this.elementWeakReference.Target as DependencyObject : null; }
            set { this.elementWeakReference = new WeakReference(value); }
        }



        /// <summary>
        /// Returns <see langword="true"/> if the behavior is attached to a region, <see langword="false"/> otherwise.
        /// </summary>
        public bool IsAttached { get; private set; }

        /// <summary>
        /// Attaches the behavior to the region.
        /// </summary>
        public void Attach()
        {
            bool isExclusion = this.GetType() == typeof(DelayedRegionCreationBehavior);
            //bool isExclusion =
            //    this.GetType() == typeof(DelayedRegionCreationBehavior) ||
            //    this.GetType() == typeof(RegionActiveAwareBehavior) ||
            //    this.GetType() == typeof(AutoPopulateRegionBehavior) ||
            //    this.GetType() == typeof(BindRegionContextToDependencyObjectBehavior);
            
                
            if (this.region == null && !isExclusion)
            {
                throw new InvalidOperationException(ResourceHelper.RegionBehaviorAttachCannotBeCallWithNullRegion);
            }

            IsAttached = true && !isExclusion;
            OnAttach();
        }

        /// <summary>
        /// Override this method to perform the logic after the behavior has been attached.
        /// </summary>
        protected abstract void OnAttach();
    }
}
