﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace System
{
    public static class PropertyExtensions
    {
        /// <summary>
        /// Returns property name from expression
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="property"></param>
        /// <returns></returns>
        public static string GetPropertyName<T>(this Expression<Func<T>> property)
        {
            var expression = property.Body as MemberExpression;

            return expression.Member.Name;
        }

        /// <summary>
        /// Return property name from expression.
        /// </summary>
        /// <example>
        /// <![CDATA[
        ///     Expression<Func<Item, object>> expression = i => i.Name;
        ///     var propertyName = expression.GetPropertyName(); // propertyName = "Name"
        /// ]]>
        /// </example>
        /// <typeparam name="T"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="expression"></param>
        /// <returns></returns>
        public static string GetPropertyName<T, TValue>(this Expression<Func<T, TValue>> expression)
        {
            var lambda = expression as LambdaExpression;

            MemberExpression memberExpression;

            if (lambda.Body is UnaryExpression)
            {
                var unaryExpression = lambda.Body as UnaryExpression;
                memberExpression = unaryExpression.Operand as MemberExpression;
            }
            else
            {
                memberExpression = lambda.Body as MemberExpression;
            }

            if (memberExpression != null)
            {
                return memberExpression.Member.Name;
            }

            return null;
        }


    }

}
