﻿using System.Services;
using System.Threading.Tasks;
using Windows.Devices.Sensors;

namespace System.Sensors
{
    public interface IGyrometerSensorService : ISensorService<Gyrometer, GyrometerReading>
    {
    }

    [ExportService("GyrometerSensorService", "", typeof(IGyrometerSensorService), int.MaxValue)]
    public class GyrometerSensorService : SensorServiceBase<Gyrometer, GyrometerReading>, IGyrometerSensorService
    {
        public GyrometerSensorService()
        {
        }

        public override void StartService()
        {
            Sensor = Gyrometer.GetDefault();
            base.StartService();
        }

        public override void Subscribe()
        {
            if (IsSensorAvailable)
                Sensor.ReadingChanged += Sensor_ReadingChanged;

            base.Subscribe();
        }

        public override void Unsubscribe()
        {
            if (IsSensorAvailable)
                Sensor.ReadingChanged -= Sensor_ReadingChanged;

            base.Unsubscribe();
        }

        void Sensor_ReadingChanged(Gyrometer sender, GyrometerReadingChangedEventArgs args)
        {
            if (EventAggregator != null)
                EventAggregator.Publish<GyrometerServiceReadingMessage>(new GyrometerServiceReadingMessage(this, args.Reading));
        }

        protected override async Task<GyrometerReading> GetSensorData()
        {
            return Sensor.GetCurrentReading();
        }

        public override uint GetMinimumReportInterval()
        {
            return Sensor.MinimumReportInterval;
        }

        public override uint GetReportInterval()
        {
            return Sensor.ReportInterval;
        }

        public override void SetReportInterval(uint value)
        {
            Sensor.ReportInterval = value;
        }
    }

    public class GyrometerServiceReadingMessage : SensorReadingMessageBase<GyrometerReading>
    {
        public GyrometerServiceReadingMessage(object sender, GyrometerReading value)
            : base(sender, value)
        {
        }
    }
}
