﻿using System;

namespace System.Windows.Input
{
    public class DelegateCommand : CommandBase
    {
        private readonly Func<bool> _canExecuteHandler;
        private readonly Action _executeHandler;

        public DelegateCommand(Action executeHandler)
            : this(executeHandler, null, true) { }

        public DelegateCommand(Action executeHandler, bool isActive)
            : this(executeHandler, null, isActive) { }

        public DelegateCommand(Action executeHandler, Func<bool> canExecuteHandler)
		    : this(executeHandler, canExecuteHandler, true) { }

        public DelegateCommand(Action executeHandler, Func<bool> canExecuteHandler, bool isActive)
            : base(isActive)
        {
            _executeHandler = executeHandler;
            _canExecuteHandler = canExecuteHandler;
        }

#region Overrides

        protected override bool OnCanExecute()
        {
            return _canExecuteHandler != null ? _canExecuteHandler() : true;
        }

        protected override void OnExecute()
        {
            _executeHandler();
        }

#endregion

    }
}
