﻿using System;

namespace System.Windows.Input
{
    public class DelegateCommand<T> : CommandBase<T>
    {
        
#region Declarations

        private readonly Func<T, bool> _canExecuteHandler;
        private readonly Action<T> _executeHandler;
        
#endregion

        public DelegateCommand(Action<T> executeHandler)
            : this(executeHandler, null, true) { }

        public DelegateCommand(Action<T> executeHandler, bool isActive)
            : this(executeHandler, null, isActive) { }

        public DelegateCommand(Action<T> executeHandler, Func<T, bool> canExecuteHandler)
		    : this(executeHandler, canExecuteHandler, true) { }

        public DelegateCommand(Action<T> executeHandler, Func<T, bool> canExecuteHandler, bool isActive) 
            : base(isActive)
        {

            _executeHandler = executeHandler;
            _canExecuteHandler = canExecuteHandler;
        }

#region Override

        protected override bool OnCanExecute(T parameter)
        {
            return _canExecuteHandler != null ? _canExecuteHandler(parameter) : true;
        }

        protected override void OnExecute(T parameter)
        {
            _executeHandler(parameter);
        }

#endregion

    }

}
